import os

def install_font(src_path):

    dr = os.path.expanduser('~/.fonts')

    # create
    try:
        os.mkdir(dr)
    except FileExistsError:
        pass

    # move files
    os.rename(src_path, dr+src_path.split('/')[-1])


import requests, zipfile, os, argparse


parser = argparse.ArgumentParser()
parser.add_argument('url', help="URL of the font")
parser.add_argument('-k','--keep', help="Keep files", action="store_true")
parser.add_argument('-v', '--verbose', help="Output execution details", action="store_true")
args = parser.parse_args()

if os.name == 'nt':
    from font_installer.windows import install_font
elif os.name == 'posix':
    from font_installer.posix import install_font
else:
    print("OS not supported!")
    exit()


def download(url, chunk_size=128):
    if 'www.dafont.com' in url:
        font_name = url.replace("-","_").split("www.dafont.com/")[1].split('.font')[0]
        dl_url = "https://dl.dafont.com/dl/?f={}".format(font_name)
    elif 'fonts.google.com' in url:
        font_name = url.replace("+","%20").split("fonts.google.com/specimen/")[1]
        dl_url = "https://fonts.google.com/download?family={}".format(font_name)
    r = requests.get(dl_url, stream=True)
    
    if args.verbose: print("Saving ZIP..")
    # save ZIP
    with open(font_name+".zip", 'wb') as f:
        for chunk in r.iter_content(chunk_size=chunk_size):
            f.write(chunk)

    if args.verbose: print("Extracting from ZIP..")
    # extract ZIP
    with zipfile.ZipFile(font_name+".zip", 'r') as zip_ref:
        zip_ref.extractall(font_name)
    
    if args.verbose: print("Installing..")
    # install
    try:
        for f in os.listdir(font_name):
            if '.otf' in f.lower() or '.ttf' in f.lower():
                install_font(os.path.abspath(font_name+"/"+f))
    
    except PermissionError:
        print("Permission Error thrown. Did you run as Administrator?")

    # delete files
    if not args.keep:
        if args.verbose: print("Deleting files..")
        os.remove(font_name+".zip")
        try:
            os.rmdir(font_name)
        except OSError:
            for f in os.listdir(font_name):
                os.remove(font_name+"/"+f)
            os.rmdir(font_name)

def main():
    download(args.url)


""" Python 3 Windows font installer
Script must be run with the privileges in order to access Windows fonts directory. System reboot is not necessary.
This will install the font and will inform programs that a new font has been added.
Python3 Windows font installer is a free python script: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
This script is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
See COPYING for a copy of the GNU General Public License.
If not, see <http://www.gnu.org/licenses/>.
Copyright (C) 2017  Lahiru Pathirage <lpsandaruwan@gmail.com> 20/02/17
"""

import ctypes
import os
import shutil
import sys

from ctypes import wintypes

try:
    import winreg
except ImportError:
    import _winreg as winreg

user32 = ctypes.WinDLL('user32', use_last_error=True)
gdi32 = ctypes.WinDLL('gdi32', use_last_error=True)

FONTS_REG_PATH = r'Software\Microsoft\Windows NT\CurrentVersion\Fonts'

HWND_BROADCAST = 0xFFFF
SMTO_ABORTIFHUNG = 0x0002
WM_FONTCHANGE = 0x001D
GFRI_DESCRIPTION = 1
GFRI_ISTRUETYPE = 3

if not hasattr(wintypes, 'LPDWORD'):
    wintypes.LPDWORD = ctypes.POINTER(wintypes.DWORD)

user32.SendMessageTimeoutW.restype = wintypes.LPVOID
user32.SendMessageTimeoutW.argtypes = (
    wintypes.HWND,   # hWnd
    wintypes.UINT,   # Msg
    wintypes.LPVOID, # wParam
    wintypes.LPVOID, # lParam
    wintypes.UINT,   # fuFlags
    wintypes.UINT,   # uTimeout
    wintypes.LPVOID  # lpdwResult
)

gdi32.AddFontResourceW.argtypes = (
    wintypes.LPCWSTR,) # lpszFilename

# http://www.undocprint.org/winspool/getfontresourceinfo
gdi32.GetFontResourceInfoW.argtypes = (
    wintypes.LPCWSTR, # lpszFilename
    wintypes.LPDWORD, # cbBuffer
    wintypes.LPVOID,  # lpBuffer
    wintypes.DWORD)   # dwQueryType


def install_font(src_path):
    # copy the font to the Windows Fonts folder
    dst_path = os.path.join(
        os.environ['SystemRoot'], 'Fonts', os.path.basename(src_path)
    )
    shutil.copy(src_path, dst_path)

    # load the font in the current session
    if not gdi32.AddFontResourceW(dst_path):
        os.remove(dst_path)
        raise WindowsError('AddFontResource failed to load "%s"' % src_path)

    # notify running programs
    user32.SendMessageTimeoutW(
        HWND_BROADCAST, WM_FONTCHANGE, 0, 0, SMTO_ABORTIFHUNG, 1000, None
    )

    # store the fontname/filename in the registry
    filename = os.path.basename(dst_path)
    fontname = os.path.splitext(filename)[0]

    # try to get the font's real name
    cb = wintypes.DWORD()
    if gdi32.GetFontResourceInfoW(
            filename, ctypes.byref(cb), None, GFRI_DESCRIPTION
    ):
        buf = (ctypes.c_wchar * cb.value)()
        if gdi32.GetFontResourceInfoW(
                filename, ctypes.byref(cb), buf, GFRI_DESCRIPTION
        ):
            fontname = buf.value

    is_truetype = wintypes.BOOL()
    cb.value = ctypes.sizeof(is_truetype)
    gdi32.GetFontResourceInfoW(
        filename, ctypes.byref(cb), ctypes.byref(is_truetype), GFRI_ISTRUETYPE
    )

    if is_truetype:
        fontname += ' (TrueType)'

    with winreg.OpenKey(
            winreg.HKEY_LOCAL_MACHINE, FONTS_REG_PATH, 0, winreg.KEY_SET_VALUE
    ) as key:
        winreg.SetValueEx(key, fontname, 0, winreg.REG_SZ, filename)


from setuptools import setup, find_packages

with open('README.md') as f:
    long_description = f.read()

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

setup(
    name='font_installer',
    version="0.1.2",
    packages=find_packages(),
    install_requires=requirements,

    author='Inesh Bose',
    desription='Install fonts from popular font sites!',
    long_description=long_description,
    long_description_content_type='text/markdown',
    license='GPL',
    url='https://github.com/ineshbose/font-installer',

    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: GNU General Public License (GPL)',
    ],
    keywords='font downloader google dafont',
    entry_points={
        "console_scripts": [
            "font-installer = font_installer.download:main",
        ],
    }
)